/*
Language: TypeScript
Author: Panu Horsmalahti <panu.horsmalahti@iki.fi>
Contributors: Ike Ku <dempfi@yahoo.com>
Description: TypeScript is a strict superset of JavaScript
Category: scripting
*/
import * as tslib_1 from "tslib";
import { APOS_STRING_MODE, QUOTE_STRING_MODE, BACKSLASH_ESCAPE, C_LINE_COMMENT_MODE, C_BLOCK_COMMENT_MODE, C_NUMBER_RE, RE_STARTERS_RE, REGEXP_MODE, IDENT_RE, TITLE_MODE as TITLE_MODE_, } from '../common';
var KEYWORDS = {
    keyword: 'in if for while finally var new function do return void else break catch ' +
        'instanceof with throw case default try this switch continue typeof delete ' +
        'let yield const class public private protected get set super ' +
        'static implements enum export import declare type namespace abstract ' +
        'as from extends async await',
    literal: 'true false null undefined NaN Infinity',
    built_in: 'eval isFinite isNaN parseFloat parseInt decodeURI decodeURIComponent ' +
        'encodeURI encodeURIComponent escape unescape Object Function Boolean Error ' +
        'EvalError InternalError RangeError ReferenceError StopIteration SyntaxError ' +
        'TypeError URIError Number Math Date String RegExp Array Float32Array ' +
        'Float64Array Int16Array Int32Array Int8Array Uint16Array Uint32Array ' +
        'Uint8Array Uint8ClampedArray ArrayBuffer DataView JSON Intl arguments require ' +
        'module console window document any number boolean string void Promise'
};
var TITLE_MODE = tslib_1.__assign({}, TITLE_MODE_, { begin: /[A-Za-z$_][0-9A-Za-z$_]*/ });
var PARAMS_CONTAINS = [
    C_LINE_COMMENT_MODE,
    C_BLOCK_COMMENT_MODE,
];
var PARAMS_MODE = {
    className: 'params',
    begin: /\(/, end: /\)/,
    excludeBegin: true,
    excludeEnd: true,
    keywords: KEYWORDS,
    contains: PARAMS_CONTAINS,
};
var ARROW_FUNCTION_MODE = {
    className: 'function',
    begin: '(\\(.*?\\)|' + IDENT_RE + ')\\s*=>',
    returnBegin: true,
    end: '\\s*=>',
    contains: [
        {
            className: 'params',
            variants: [
                {
                    begin: IDENT_RE
                },
                {
                    begin: /\(\s*\)/,
                },
                tslib_1.__assign({}, PARAMS_MODE, { className: undefined })
            ]
        }
    ]
};
PARAMS_CONTAINS.push(ARROW_FUNCTION_MODE);
export var TypeScript = {
    name: 'typescript',
    aliases: ['ts'],
    keywords: KEYWORDS,
    contains: [
        {
            className: 'meta',
            begin: /^\s*['"]use strict['"]/
        },
        APOS_STRING_MODE,
        QUOTE_STRING_MODE,
        {
            className: 'string',
            begin: '`', end: '`',
            contains: [
                BACKSLASH_ESCAPE,
                {
                    className: 'subst',
                    begin: '\\$\\{', end: '\\}'
                }
            ]
        },
        C_LINE_COMMENT_MODE,
        C_BLOCK_COMMENT_MODE,
        {
            className: 'number',
            variants: [
                { begin: '\\b(0[bB][01]+)' },
                { begin: '\\b(0[oO][0-7]+)' },
                { begin: C_NUMBER_RE }
            ],
            relevance: 0
        },
        {
            begin: '(' + RE_STARTERS_RE + '|\\b(case|return|throw)\\b)\\s*',
            keywords: 'return throw case',
            contains: [
                C_LINE_COMMENT_MODE,
                C_BLOCK_COMMENT_MODE,
                REGEXP_MODE,
                ARROW_FUNCTION_MODE
            ],
            relevance: 0
        },
        {
            className: 'function',
            begin: 'function', end: /[\{;]/, excludeEnd: true,
            keywords: KEYWORDS,
            contains: [
                'self',
                TITLE_MODE,
                PARAMS_MODE,
            ],
            illegal: /%/,
            relevance: 0 // () => {} is more typical in TypeScript
        },
        {
            beginKeywords: 'constructor', end: /\{/, excludeEnd: true,
            contains: [
                'self',
                PARAMS_MODE,
            ]
        },
        {
            begin: /module\./,
            keywords: { built_in: 'module' },
            relevance: 0
        },
        {
            beginKeywords: 'module', end: /\{/, excludeEnd: true
        },
        {
            beginKeywords: 'namespace', end: /\{/, excludeEnd: true,
            contains: [TITLE_MODE]
        },
        {
            beginKeywords: 'interface', end: /\{/, excludeEnd: true,
            keywords: 'interface extends',
            contains: [TITLE_MODE]
        },
        {
            begin: /\$[(.]/ // relevance booster for a pattern common to JS libs: `$(something)` and `$.something`
        },
        {
            begin: '\\.' + IDENT_RE, relevance: 0 // hack: prevents detection of keywords after dots
        },
        {
            className: 'meta', begin: '@[A-Za-z]+'
        }
    ]
};
//# sourceMappingURL=typescript.js.map