"use strict";
/*
Language: JavaScript
Category: common, scripting
*/
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var common_1 = require("../common");
var IDENT_RE = '[A-Za-z$_][0-9A-Za-z$_]*';
var KEYWORDS = {
    keyword: 'in of if for while finally var new function do return void else break catch ' +
        'instanceof with throw case default try this switch continue typeof delete ' +
        'let yield const export super debugger as async await static ' +
        // ECMAScript 6 modules import
        'import from as',
    literal: 'true false null undefined NaN Infinity',
    built_in: 'eval isFinite isNaN parseFloat parseInt decodeURI decodeURIComponent ' +
        'encodeURI encodeURIComponent escape unescape Object Function Boolean Error ' +
        'EvalError InternalError RangeError ReferenceError StopIteration SyntaxError ' +
        'TypeError URIError Number Math Date String RegExp Array Float32Array ' +
        'Float64Array Int16Array Int32Array Int8Array Uint16Array Uint32Array ' +
        'Uint8Array Uint8ClampedArray ArrayBuffer DataView JSON Intl arguments require ' +
        'module console window document Symbol Set Map WeakSet WeakMap Proxy Reflect ' +
        'Promise'
};
var NUMBER = {
    className: 'number',
    variants: [
        { begin: '\\b(0[bB][01]+)' },
        { begin: '\\b(0[oO][0-7]+)' },
        { begin: common_1.C_NUMBER_RE }
    ],
    relevance: 0
};
var SUBST = {
    className: 'subst',
    begin: '\\$\\{', end: '\\}',
    keywords: KEYWORDS,
    contains: [] // defined later
};
var TEMPLATE_STRING = {
    className: 'string',
    begin: '`', end: '`',
    contains: [
        common_1.BACKSLASH_ESCAPE,
        SUBST
    ]
};
SUBST.contains = [
    common_1.APOS_STRING_MODE,
    common_1.QUOTE_STRING_MODE,
    TEMPLATE_STRING,
    NUMBER,
    common_1.REGEXP_MODE
];
var PARAMS_CONTAINS = SUBST.contains.concat([
    common_1.C_BLOCK_COMMENT_MODE,
    common_1.C_LINE_COMMENT_MODE
]);
exports.JavaScript = {
    name: 'javascript',
    aliases: ['js', 'jsx'],
    keywords: KEYWORDS,
    contains: [
        {
            className: 'meta',
            relevance: 10,
            begin: /^\s*['"]use (strict|asm)['"]/
        },
        {
            className: 'meta',
            begin: /^#!/, end: /$/
        },
        common_1.APOS_STRING_MODE,
        common_1.QUOTE_STRING_MODE,
        TEMPLATE_STRING,
        common_1.C_LINE_COMMENT_MODE,
        common_1.C_BLOCK_COMMENT_MODE,
        NUMBER,
        {
            begin: /[{,]\s*/, relevance: 0,
            contains: [
                {
                    begin: IDENT_RE + '\\s*:', returnBegin: true,
                    relevance: 0,
                    contains: [{ className: 'attr', begin: IDENT_RE, relevance: 0 }]
                }
            ]
        },
        {
            begin: '(' + common_1.RE_STARTERS_RE + '|\\b(case|return|throw)\\b)\\s*',
            keywords: 'return throw case',
            contains: [
                common_1.C_LINE_COMMENT_MODE,
                common_1.C_BLOCK_COMMENT_MODE,
                common_1.REGEXP_MODE,
                {
                    className: 'function',
                    begin: '(\\(.*?\\)|' + IDENT_RE + ')\\s*=>', returnBegin: true,
                    end: '\\s*=>',
                    contains: [
                        {
                            className: 'params',
                            variants: [
                                {
                                    begin: IDENT_RE
                                },
                                {
                                    begin: /\(\s*\)/,
                                },
                                {
                                    begin: /\(/, end: /\)/,
                                    excludeBegin: true, excludeEnd: true,
                                    keywords: KEYWORDS,
                                    contains: PARAMS_CONTAINS
                                }
                            ]
                        }
                    ]
                },
                {
                    begin: /</, end: /(\/\w+|\w+\/)>/,
                    subLanguage: 'xml',
                    contains: [
                        { begin: /<\w+\s*\/>/, skip: true },
                        {
                            begin: /<\w+/, end: /(\/\w+|\w+\/)>/, skip: true,
                            contains: [
                                { begin: /<\w+\s*\/>/, skip: true },
                                'self'
                            ]
                        }
                    ]
                }
            ],
            relevance: 0
        },
        {
            className: 'function',
            beginKeywords: 'function', end: /\{/, excludeEnd: true,
            contains: [
                tslib_1.__assign({}, common_1.TITLE_MODE, { begin: IDENT_RE }),
                {
                    className: 'params',
                    begin: /\(/, end: /\)/,
                    excludeBegin: true,
                    excludeEnd: true,
                    contains: PARAMS_CONTAINS
                }
            ],
            illegal: /\[|%/
        },
        {
            begin: /\$[(.]/ // relevance booster for a pattern common to JS libs: `$(something)` and `$.something`
        },
        common_1.METHOD_GUARD,
        {
            className: 'class',
            beginKeywords: 'class', end: /[{;=]/, excludeEnd: true,
            illegal: /[:"\[\]]/,
            contains: [
                { beginKeywords: 'extends' },
                common_1.UNDERSCORE_TITLE_MODE
            ]
        },
        {
            beginKeywords: 'constructor', end: /\{/, excludeEnd: true
        }
    ],
    illegal: /#(?!!)/
};
//# sourceMappingURL=javascript.js.map